#include "../config.h"

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>		// for O_RDONLY

#include <time.h>		// for localtime
#include <setjmp.h>

#include <qdir.h> 
#include <qfileinf.h> 

#include "packageInfo.h"
#include "debInterface.h"
#include "kpackage.h"
#include "utils.h"
#include "procbuf.h"


procbuf reader;

#define STATUS          "/var/lib/dpkg/status"
#define INFODIR         "/var/lib/dpkg/info/"

static param pinstall[] =  {
  {"Allow Downgrade",TRUE,TRUE},
  {"Check Conflicts",TRUE,TRUE},
  {"Check Dependencies",TRUE,TRUE},
  {"Test (do not install)",FALSE,FALSE},
  {0,FALSE,FALSE}
};

static param puninstall[] =  {
  {"Purge Config Files",FALSE,FALSE},
  {"Check Dependencies",TRUE,TRUE},
  {"Test (do not uninstall)",FALSE,FALSE},
  {0,FALSE,FALSE}
};

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
DEB::DEB():pkgInterface()
{
  head = "DEB/";
  icon = "deb.xpm";  
  pict = new QPixmap();
  *pict = globalKIL->loadIcon(icon); 
  bad_pict = new QPixmap();
  *bad_pict = globalKIL->loadIcon("bdeb.xpm");
  pinstall[0].name = klocale->translate("Allow Downgrade");
  pinstall[1].name = klocale->translate("Check Conflicts");
  pinstall[2].name = klocale->translate("Check Dependencies");
  pinstall[3].name = klocale->translate("Test (do not install)");

  puninstall[0].name = klocale->translate("Purge Config Files");
  puninstall[1].name = klocale->translate("Check Dependencies");
  puninstall[2].name = klocale->translate("Test (do not uninstall)");
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
DEB::~DEB()
{;
}

//////////////////////////////////////////////////////////////////////////////
param *DEB::initinstallOptions()
{
  return &(pinstall[0]);
}

param *DEB::inituninstallOptions()
{
  return &(puninstall[0]);
}

// check if debian file
bool DEB::isType(char *buf)
{
  if  (!strcmp(buf,"!<arch>\n")) {
    return true;
  } else if (!strncmp(buf,"0.9",3)) {
    return true;
  } else
    return false;
}

void DEB::listInstalled(LpackageInfo *pki)
{
  int percentage;
  int np;
  QString vb;
  char linebuf[1024];
  FILE *file;
  bool installed = false;

  kpkg->kp->setStatus(klocale->translate("Querying DEB database for installed packages"));
  percentage=0;
  kpkg->kp->setPercent(percentage);

  np = 0;
  file= fopen(STATUS,"r");
  vb = "";

  if (file) {
    while (fgets(linebuf,sizeof(linebuf),file)) {
      if (strcmp(linebuf,"\n")) {
	if (!strncmp(linebuf,"Status:",7)) {
	  if (strncmp(linebuf+strlen(linebuf)-15," not-installed\n",15)) {
	    installed = true;
	  }
	}
	vb += linebuf;
      } else {
	if (installed) 
	  pki->inSort(collectInfo(vb.data()));
	installed = false;
	vb.truncate(0);
      }
    }
    fclose(file);
  }
  kpkg->kp->setPercent(100);
}


//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// mode: i = query installed    u = query uninstalled
packageInfo *DEB::getPackageInfo(char mode, const char *name)
{
  packageInfo *pki = 0; 
  QString vb,search;
  char linebuf[1024];
  FILE *file;
  bool found = false;
  
  switch(mode)
    {
      ////////////////////////////////////////////////////////////////////////
      // query an installed package!
    case 'i':
      file= fopen(STATUS,"r");
      vb = "";
      search = "Package: ";
      search += name;
      search += "\n";

      if (file) {
	while (fgets(linebuf,sizeof(linebuf),file)) {
	  if (!found) {
	    if (!strcmp(search.data(),linebuf)) {
	      found = true;
	    } else {
	      continue;
	    }
	  }
	  if (strcmp(linebuf,"\n")) {
	    vb += linebuf;
	  } else {
	    pki = DEB::collectInfo(vb.data());
	    break;	
	  }
	}
      }
      fclose(file);
      break;

      ////////////////////////////////////////////////////////////////////
      // query an uninstalled package      
    case 'u':
      reader.setup("dpkg");
      *reader.proc << "--info"<<name;
      if (reader.start(0))
	pki = DEB::collectInfo(reader.buf.data());

      break;
    }
  return pki;
}


//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
packageInfo *DEB::collectInfo(char *inp)
{
  QDict<QString> *a = new QDict<QString>;
  a->setAutoDelete(TRUE);

  int have_group;
  char *str, *xstr;
  QString qstr;
  bool bad_install = FALSE;

    have_group = 0;

    //   remove prefix output from dpkg
    str = strtok(inp,"\n");
    do {
      xstr = strchr(str,':');
      if (*str == ' ')
	str++;
      if (!strncmp("Package",str,7))
	break;
    } while ((str = strtok(NULL,"\n")));

    // parse 'name: text' elements
    if (str) {
      do {
	xstr = strchr(str,':');
	if (xstr) {
	  *xstr++ = 0;
	  xstr++;
	  
	  if (*str == ' ')
	    str++;
	  if (!strcmp("Conffiles",str)) {
	  } else if (!strcmp("Description",str)) {
	    qstr = xstr;
	    qstr += "\n";
	    break;
	  } else if (!strcmp("Package",str)) {
	    a->insert("name", new QString(xstr));
	  } else if (!strcmp("Section",str)) {
	    a->insert("group", new QString(xstr));
	    have_group++;
	  } else if (!strcmp("Status",str)) {
	    if (strcmp("install ok installed",xstr) && 
		strcmp("deinstall ok installed",xstr)) {
	      bad_install = TRUE;
	    }
	    a->insert(str, new QString(xstr));
	  } else if (!strcmp("Installed-Size",str)) {
	    a->insert("size", new QString(xstr));
	  } else {
	    a->insert(str, new QString(xstr));
	  }
	}
      } while ((str = strtok(NULL,"\n")));
    }
    
    // parse description text
    while ((str = strtok(NULL,"\n"))) {
       qstr += str;
    }
    //     qstr += '\n';
     a->insert("Description", new QString(qstr.data()));

    if (!have_group) {
	  a->insert("group", new QString("OTHER"));
    }
    packageInfo *i = new packageInfo(a,this);
    i->bad_install = bad_install;
    return i;

}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
QList<char> *DEB::getFileList(packageInfo *p)
{
  char *filelistfile;
  QString vb;
  char *name;
  char mode;
  char linebuf[1024];
  FILE *file;
  char *s, *str, *strp;
  int prlen;

  if(p->getFilename().data())
    mode = 'u';
  else
    mode = 'i';

  QList<char> *filelist = new QList<char>;
  filelist->setAutoDelete(TRUE);

  switch(mode)
    {
      ////////////////////////////////////////////////////////////////////////
      // query an installed package!
    case 'i':
      name = p->getProperty("name")->data();

      vb = INFODIR;
      vb += name;
      vb += ".list";
      filelistfile= vb.data();
      file= fopen(filelistfile,"r");

      if (file) {
	while (fgets(linebuf,sizeof(linebuf),file)) {
	  s = new char[strlen(linebuf)+1];
	  strcpy(s,linebuf);
	  filelist->append(s);
	}
	fclose(file);
      }
      break;

      ////////////////////////////////////////////////////////////////////
      // query an uninstalled package      
    case 'u':
      name = p->getFilename().data();
      reader.setup("dpkg");
      *reader.proc << "--contents"<<name;

      if (!reader.start(1))
	return 0;

      str = strtok(reader.buf.data(),"\n");
      strp = strrchr(str,' ');
      if (strp) {
	prlen = strp - str;
      } else {
	prlen = 0;
      }
      do {
	s = new char[strlen(str)+1];
	strcpy(s,str+prlen);
		filelist->append(s);
      } while ((str = strtok(NULL,"\n")));
      break;
    }
   
  return filelist;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
QList<char> *DEB::depends(const char *name, int src){}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
QList<char> *DEB::verify(packageInfo *package, QList<char> *files)
{
  char *ln;
  int n, p = 0;
  uint c = 0;
  QList<char> *errorlist = new QList<char>;
  QDir d;

  package = package; //silence compiler

  QListIterator<char> it(*files);
  uint step = (it.count() / 100) + 1;

  kpkg->kp->setStatus(klocale->translate("Verifying"));
  kpkg->kp->setPercent(0);


  char *s;
  for(; (s = it.current()); ++it) {

    // Update the status progress
    c++;
    if(c > step) {
      c=0; p++;
      kpkg->kp->setPercent(p);
    }

    n = strlen(s);
    s[n-1] = 0;
    if (!d.exists(s)) {
      ln = new char[n];
      strcpy(ln,s);
      errorlist->append(ln);
    }	
  }
  kpkg->kp->setPercent(100);
  return errorlist;
}

//////////////////////////////////////////////////////////////////////////////
// Use the ~/.dpkg_ok file to tell if dpkg has worked or failed
//////////////////////////////////////////////////////////////////////////////
int DEB::dpkgChange()
{
    QDateTime st;
    st.setTime_t(0);

    char *tmpd = getenv("HOME");
    if (tmpd) {
      QString tfile;
      tfile.sprintf("%s/.dpkg_ok",tmpd);
      QFileInfo *fileInfo = new QFileInfo(tfile.data());
      if (fileInfo->exists()) {
	return st.secsTo(fileInfo->lastModified());
      } else {
	return 0;
      }
    } else
      return 0;
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
int DEB::uninstall(int uninstallFlags, packageInfo *p)
{
  char *name;
  int test = 0, ctime, nctime;

  ctime = dpkgChange();
  name = p->getProperty("name")->data();

  reader.setup("kvt");
  *reader.proc << "-e" << "kpackage_dpkg_rm";

  if ((uninstallFlags>>0 & 1) ^ puninstall[0].invert)
    *reader.proc << "--purge";
  else
    *reader.proc << "--remove";
  if ((uninstallFlags>>1 & 1) ^ puninstall[1].invert)
    *reader.proc << "--force-depends";
  if ((uninstallFlags>>2 & 1) ^ puninstall[2].invert) {
    *reader.proc << "--no-act";
    test = 1;
  }
  *reader.proc <<  name;

  reader.start(1);
  nctime = dpkgChange();
  
  return (nctime == ctime) || test;
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
int DEB::install(int installFlags, packageInfo *p)
{
  char *name;
  int test = 0, ctime, nctime;

  ctime = dpkgChange();
  name = p->getFilename().data();
  reader.setup("kvt");
  *reader.proc << "-e" << "kpackage_dpkg_ins";
  if ((installFlags>>0 & 1) ^ pinstall[0].invert)
    *reader.proc << "--refuse-downgrade";
  if ((installFlags>>1 & 1) ^ pinstall[1].invert)
    *reader.proc << "--force-conflicts";
  if ((installFlags>>2 & 1) ^ pinstall[2].invert)
    *reader.proc << "--force-depends";
  if ((installFlags>>3 & 1) ^ pinstall[3].invert) {
    *reader.proc << "--no-act";
    test = 1;
  }
  *reader.proc <<  name;

  reader.start(1);
  nctime = dpkgChange();
  
  return (nctime == ctime) || test;
}






