//////////////////////////////////////////////////////////////////////
// File  : kpackage.cpp
// Author: Damyan Pepper
//
// See kpackage.h for more information.
//////////////////////////////////////////////////////////////////////

#include "../config.h"
extern "C"
{
#include <time.h>
}  
#include <unistd.h>

#include <qdir.h>
#include <kurl.h>
#include <kapp.h>

#include "kpackage.h"
#include "kpackage.moc"
#include "aboutDialog.h"
#include "managementWidget.h"
#include "pkgInterface.h"

extern KApplication *app;

// Constructor
KPKG::KPKG(KConfig *_config, const char *name) 
  : KTopLevelWidget(name)
{

  kp = new KPACKAGE(_config,this);
  kp->show();
  setView(kp);

  readSettings();
  // Get a nice default size
  resize(600,400);

  // Setup the menu
  setupMenu();

  recent_files.setAutoDelete(TRUE);

  recentpopup->clear();

  for ( int i =0 ; i < (int)recent_files.count(); i++){
    recentpopup->insertItem(recent_files.at(i));
  }
  prop_restart = false;
}

// Destructor
KPKG::~KPKG()
{
  delete menubar;
  delete filemenu;
  delete helpmenu;
  delete recentpopup;
}

// Set up the menu
void KPKG::setupMenu()
{
  recentpopup = new QPopupMenu ();

  filemenu = new QPopupMenu();
  filemenu->insertSeparator();
  filemenu->insertItem(klocale->translate("&Open"),kp , SLOT(fileOpen()));
  filemenu->insertItem(klocale->translate("Open &URL"), kp, SLOT(fileOpenUrl()));
  filemenu->insertItem (klocale->translate("Open &Recent..."), recentpopup);
  connect( recentpopup, SIGNAL(activated(int)), SLOT(openRecent(int)) );
  filemenu->insertItem(klocale->translate("&Find..."), kp, SLOT(find()));
  filemenu->insertItem(klocale->translate("&Quit"), kp, SLOT(fileQuit()));

  helpmenu = new QPopupMenu();
  helpmenu->insertItem(klocale->translate("&About..."), kp, SLOT(helpAbout()));
  helpmenu->insertItem(klocale->translate("&Index..."), kp, SLOT(helpHelp()));


  menubar = new KMenuBar(this);
  menubar->insertItem(klocale->translate("&File"),filemenu);
  menubar->insertSeparator();
  menubar->insertItem(klocale->translate("&Help"),helpmenu);

  setMenu(menubar);

  urlList.setAutoDelete(TRUE);
}

void KPKG::disableMenu()
{
 filemenu->setItemEnabled(filemenu->idAt(1),FALSE);
 filemenu->setItemEnabled(filemenu->idAt(2),FALSE);
 filemenu->setItemEnabled(filemenu->idAt(3),FALSE);
 filemenu->setItemEnabled(filemenu->idAt(4),FALSE);
}

void KPKG::enableMenu()
{
 filemenu->setItemEnabled(filemenu->idAt(1),TRUE);
 filemenu->setItemEnabled(filemenu->idAt(2),TRUE);
 filemenu->setItemEnabled(filemenu->idAt(3),TRUE);
 filemenu->setItemEnabled(filemenu->idAt(4),TRUE);
}

void KPKG::openRecent(int i){
  kp->openNetFile( recent_files.at(i) );	  
}

void KPKG::add_recent_file(const char* newfile){
  if(recent_files.find(newfile) != -1)
    return; // it's already there

  if( recent_files.count() < 5)
    recent_files.insert(0,newfile);
  else{
    recent_files.remove(4);
    recent_files.insert(0,newfile);
  }

  recentpopup->clear();

  for ( int i =0 ; i < (int)recent_files.count(); i++){
    recentpopup->insertItem(recent_files.at(i));
  }
}

void KPKG::readSettings() {
  QString str;
  KConfig *config = app->getConfig();
  config->setGroup("Recently_Opened_Files");

  str = config->readEntry("1","");
  if (!str.isEmpty())
    recent_files.append(str.data());

  str = config->readEntry("2","");
  if (!str.isEmpty())
    recent_files.append(str.data());

  str = config->readEntry("3","");
  if (!str.isEmpty())
    recent_files.append(str.data());

  str = config->readEntry("4","");
  if (!str.isEmpty())
    recent_files.append(str.data());

  str = config->readEntry("5","");
  if (!str.isEmpty())
    recent_files.append(str.data());
}

void KPKG::writeSettings(){
  KConfig *config = app->getConfig();
  config->setGroup("Recently_Opened_Files");

  QString recent_number;
  for(int i = 0; i <(int) recent_files.count();i++){
    recent_number.setNum(i+1);
    config->writeEntry(recent_number.data(),recent_files.at(i));
  }	
  config->sync();
}
		
void KPKG::saveProperties(KConfig *config )
{
    config->writeEntry("Name", kp->save_url);
}


void KPKG::readProperties(KConfig *config)
{
    QString entry = config->readEntry("Name"); // no default
    if (entry.isNull())
	return;
    kp->openNetFile(entry.data());
    prop_restart = true;
}

void KPKG::closeEvent ( QCloseEvent *) {
    delete menubar;
    kp->fileQuit();
}

// Constructor
KPACKAGE::KPACKAGE(KConfig *_config, QWidget *parent, const char *name) 
  : QWidget(parent,name)
{

  // Save copy of config
  config = _config;

  // Setup the dropzone
  setupDropzone();

  // Setup the mode widgets
  setupModeWidgets();

  // Setup the status bar
  setupStatusBar();

  file_dialog = NULL;
  kfm = NULL;
  srchdialog = NULL;  

}

// Destructor
KPACKAGE::~KPACKAGE()
{
}

// resize event -- arrange the widgets
void KPACKAGE::resizeEvent(QResizeEvent *re)
{
  re = re;			// prevent warning
  arrangeWidgets();
}

// Set up the drop zone
void KPACKAGE::setupDropzone()
{
  dropzone = new KDNDDropZone(this, DndURL);
  connect(dropzone,SIGNAL(dropAction(KDNDDropZone*)),
	  SLOT(dropAction(KDNDDropZone*)));
}

// Set up the mode widgets
void KPACKAGE::setupModeWidgets()
{
  int i;

  management = new managementWidget(this);
  for (i = 0; i < kpinterfaceN; i++) {
    kpinterface[i]->installation =
      new installationWidget(this, kpinterface[i]);
    connect(kpinterface[i]->installation,
	    SIGNAL(finished(int, pkgInterface *,int)),
	    SLOT(modeFinished(int, pkgInterface *,int)));
  }
}

// Set up the status bar
void KPACKAGE::setupStatusBar()
{
  statusbar = new QFrame(this);
  statusbar->setFrameStyle(QFrame::Raised | QFrame::Panel);
  processProgress = new KProgress(0,100,100,KProgress::Horizontal,statusbar);
  processProgress->setBarStyle(KProgress::Solid);
  processProgress->setTextEnabled(FALSE);

  status = new QLabel(klocale->translate("Management Mode"), statusbar);
  status->setFont(QFont("Helvetica",10, QFont::Normal));
}
  
// Arrange the widgets nicely
void KPACKAGE::arrangeWidgets()
{
  int i;

  statusbar->resize(width(),20);
  statusbar->move(0,height()-20);
  status->resize((statusbar->width() / 4) * 3, 16);
  status->move(2,2);
  processProgress->resize(statusbar->width() / 4 - 4, 16);
  processProgress->move((statusbar->width() / 4) * 3 + 3, 2);
  
  management->resize(width(),height() - 20);

  for (i = 0; i < kpinterfaceN; i++) 
    kpinterface[i]->installation->resize(width(),height() - 20);
}

// Remove tempfile if needed
void KPACKAGE::rmtemp()
{
  if (tmpFile.length() > 0) {
    unlink(tmpFile.data());
    tmpFile.truncate(0);
  }
}

// Change the mode
void KPACKAGE::setMode(int newmode, pkgInterface *type, int refresh) 
{ 
  int i;

  if(newmode == Management)	// entering management mode so...
    {
      for (i = 0; i < kpinterfaceN; i++)
	kpinterface[i]->installation->hide();
      management->show();	// show the management widget
      kpkg->enableMenu();
      management->collectData(refresh); // refresh the management widget
    }
  else				// entering installation mode so...
    {
      for (i = 0; i < kpinterfaceN; i++) {// hide all except the 
	if (kpinterface[i] == type)       // correct install widget
	  kpinterface[i]->installation->show();
	else
	  kpinterface[i]->installation->hide();
      }
      kpkg->disableMenu();
      
      management->hide();	// hide the management widget
    }
}

void KPACKAGE::fileQuit()		// file->quit selected from menu
{
  kpkg->writeSettings();
  rmtemp();
  KApplication::exit(0);	// exit the application
}

void KPACKAGE::fileOpen()		// file->quit selected from menu
{
    QString fname;
    QFileDialog *box;
    pkgInterface *type;

    box = getFileDialog(klocale->translate("Select Document to Open"));
    
    box->show();
    
    if (!box->result())   /* cancelled */
      return;
    if(box->selectedFile().isEmpty()) {  /* no selection */
      return;
    }
    

    fname =  box->selectedFile();
    save_url = fname;
    kpkg->add_recent_file(fname.data());
    type = pkType(fname.data());
    if (!type)
      return;
    setMode(Installation,type,0);	// enter installation mode
    installPackage(fname.data(),type);	// start installing this package
}

pkgInterface *KPACKAGE::pkType(const char *fname)
{
  // Get the package information for this package
  char buf[50];
  int i;

  FILE *file= fopen(fname,"r");
  if (file) {
    fgets(buf,sizeof(buf),file);

    for (i = 0; i < kpinterfaceN; i++) {
      if (kpinterface[i]->isType(buf)) {
	return kpinterface[i];
      }
    }
  }

  KpMsgE(klocale->translate("Unknown package type: %s"),fname,TRUE);
  return 0;
}

void KPACKAGE::openNetFile( const char *_url )
{
    
  QString string;


  kpkg->add_recent_file(_url);
  pkgInterface *type;

  netFile = _url;
  save_url = _url;

  KURL *u = new KURL( netFile.data() );
  if ( u->isMalformed() )
    {
      delete u; // we need to re-create the URL in order 
      // to perform a new check for malforming

      if (netFile.data()[0] == '/'){
	// absolute path
	u=new KURL( QString(QString("file:")+QString(netFile.data())) );
      }
      else{
	u=new KURL( QString(QString("file:")+
			    QDir::currentDirPath()+"/"+QString(netFile.data())) );
      }
      if (u->isMalformed()){
	KpMsgE(klocale->translate("Malformed URL: %s"),netFile.data(),TRUE);
	delete u;
	return;
      }
    }

  // Just a usual file ?
  if ( strcmp( u->protocol(), "file" ) == 0 )
    {
      QString string;
      string.sprintf(klocale->translate("Loading '%s'"),u->path() );
      setStatus(string.data());

      type = pkType(u->path());
      if (!type)
	return;
      setMode(Installation,type,0);	// enter installation mode
      installPackage(u->path(),type);	// start installing this package

      setStatus(klocale->translate("Done"));
      delete u;
      return;
    }
    
  if ( kfm != 0L )
    {
      KpMsgE(	    klocale->translate("KXXX is already waiting\n"\
	    "for an internet job to finish\n"\
	    "Please wait until has finished.\n"\
	    "Alternatively stop the running one."),
	    "",TRUE);
      return;
    }
    
  setStatus(klocale->translate("Calling KFM"));

  kfm = new KFM;
  kfm->allowKFMRestart(TRUE);
  setStatus(klocale->translate("KFM Done"));

  if ( !kfm->isOK() )
    {
      KpMsgE(	    klocale->translate("Could not start or find KFM"),"",TRUE);
      delete kfm;
      kfm = 0L;
      return;
    }
    
  setStatus(klocale->translate("Starting Job"));
  rmtemp(); // remove old tmp file

  char *tmpd = getenv("HOME");
  if (!tmpd)
    tmpd = "/tmp";
  tmpFile.sprintf( "%s/%i.kpkg", tmpd, time( 0L ) );
  connect( kfm, SIGNAL( finished() ), this, SLOT( slotKFMFinished() ) );
  setStatus(klocale->translate("Connected"));
  kfm->copy( netFile.data(), tmpFile.data() );
  setStatus(klocale->translate("Waiting..."));
}

void KPACKAGE::fileOpenUrl(){

  DlgLocation geturl( klocale->translate("Open Location:"), save_url.data(), this );

  if ( geturl.exec() )
    {
      kpkg->add_recent_file(geturl.getText());
      openNetFile( geturl.getText() );
    }
}

void KPACKAGE::find(){
  if (srchdialog)
    srchdialog->show();
  else
    srchdialog = new Search(0, "searchdialog");
}

void KPACKAGE::slotKFMFinished()
{
  QString string;
  pkgInterface *type;

    string.sprintf(klocale->translate("Finished '%s'"),tmpFile.data());
    setStatus(string.data());

    KURL *u = new KURL( tmpFile.data() );

    type = pkType(u->path());
    if (!type)
      return;
    setMode(Installation,type,0);	// enter installation mode
    installPackage(u->path(),type);	// start installing this package

    disconnect( kfm, SIGNAL( finished() ), this, SLOT( slotKFMFinished() ) );

    delete kfm;
    kfm = 0L;
}

QFileDialog* KPACKAGE::getFileDialog(const char* captiontext){

  if(!file_dialog){
    file_dialog = new QFileDialog(QDir::currentDirPath().data(),"*",
				  this,"file_dialog",TRUE);
  }

  file_dialog->setCaption(captiontext);
  file_dialog->rereadDir();

  return file_dialog;
}

void KPACKAGE::helpAbout()	// help->about selected from menu
{
  aboutDialog *about = new aboutDialog(this); // create the about box
  about->exec();		// execute the about box
  delete about;			// delete the about box
}

void KPACKAGE::helpHelp()		// Help->Help selected from menu
{
      app->invokeHTMLHelp( "kpackage/index.html", "" );  
}

void KPACKAGE::dropAction(KDNDDropZone *dz) // something has been dropped
{
  char *s;
  int i = 0;

  // Call openNetFile with the first URL, save the rest to be processed
    QStrList &list = dz->getURLList();
    
    for ( s = list.first();
	  s != 0;
	  s = list.next(), i++ ) {
      if (i == 0)
	openNetFile(s);
      else
	kpkg->urlList.append(s);
    }
}

void KPACKAGE::installPackage(const char *name, pkgInterface *type)	// start installing package
{
    type->installation->installPackage(name); // so tell the installation widget about it
}

void KPACKAGE::modeFinished(int mode, pkgInterface *interface, int refresh) // a mode has finished
{				// so set the new mode appropriately
  if(mode == Installation)
    setMode(Management, interface, refresh);
  else
    setMode(Installation, interface, refresh);
}

void KPACKAGE::setStatus(const char *s)	// set the text in the status bar
{
  status->setText(s);
  kapp->processEvents();	// refresh the screen
}

char *KPACKAGE::getStatus()	// get the text in the status bar
{
  if(status)
    return (char*)status->text();
  else 
    return NULL;
}

void KPACKAGE::setPercent(int x)	// set the progress in the status bar
{
  processProgress->setValue(x);
  kapp->processEvents();	// refresh it
}

Search::Search(QWidget *parent, const char *name)
    : QDialog(parent, name,FALSE){

    this->setFocusPolicy(QWidget::StrongFocus);
    frame1 = new QGroupBox(klocale->translate("Find"), this, "frame1");

    value = new QLineEdit( this, "value");
    value->setFocus();
    connect(value, SIGNAL(returnPressed()), this, SLOT(ok_slot()));

    substr = new QCheckBox(klocale->translate("Sub string"), this, "substr");
    substr->setChecked(TRUE);
    wrap = new QCheckBox(klocale->translate("Wrap search"), this, "wrap");
    wrap->setChecked(TRUE);

    ok = new QPushButton(klocale->translate("Find"), this, "find");
    connect(ok, SIGNAL(clicked()), this, SLOT(ok_slot()));

    cancel = new QPushButton(klocale->translate("Done"), this, "cancel");
    connect(cancel, SIGNAL(clicked()), this, SLOT(done_slot()));
    //    connect(cancel, SIGNAL(clicked()), this, SLOT(reject()));

    setFixedSize(330, 130);
    show();
}

Search::~Search()
{
}

void Search::ok_slot() 
{
  int pkg;

  QString to_find = this->value->text();
  
  pkg = kpkg->kp->management->search(to_find.data(),
		 substr->isChecked(),FALSE);
  if (pkg == 0 && wrap->isChecked()) {
    pkg = kpkg->kp->management->search(to_find.data(),
		 substr->isChecked(),TRUE);
  }
  if (pkg == 0)
    KpMsg(klocale->translate("Note"),
	  klocale->translate("%s was not found"),to_find.data(),TRUE); 
}

void Search::done_slot()
{
  this->hide();
}
void Search::resizeEvent(QResizeEvent *){


    frame1->setGeometry(5, 5, width() - 10, 80);
    cancel->setGeometry(width() - 80, height() - 30, 70, 25);
    ok->setGeometry(10, height() - 30, 70, 25);
    value->setGeometry(20, 25, width() - 40, 25);
    substr->setGeometry(20, 55, 110, 25);
    wrap->setGeometry(width()- 20 - 130, 55, 130, 25);

}

