//////////////////////////////////////////////////////////////////////////
// $Id: kpackage.h,v 1.6 1997/04/01 10:52:26 root Exp $	
// File  : kpackage.h
// Author: Damyan Pepper
//
// This is the main widget for kpackage
// The whole widget is a DND drop zone where users can drop packagess to 
// be installed.
// The widget can be in one of two modes: management and installation.
//
// Management mode displays a tree list showing all the packages that are 
// currently installed on the system (each in their appropriate groups)
// and a display of the currently selected package's properties.
// 
// Installation mode displays the properties of the package to install
// and a set of options for installation
//
// The displays for each of these modes are contained within a widget.
// When the mode changes, the appropriate widget is show and the other
// hidden.
//
// KPACKAGE also holds the menu bar.  At current, the menu structure is as
// follows:
//
// File -> Quit
// Help -> About
//
// This will, of course, be expanded in future versions as more 
// functionality is added to the application.
//////////////////////////////////////////////////////////////////////////

#ifndef KPACKAGE_H
#define KPACKAGE_H

#include "../config.h"

// Standard Headers
#include <stdio.h>
#include <sys/cdefs.h>

// Qt Headers
#include <qdir.h>
#include <qwidget.h>
#include <qframe.h>
#include <qlabel.h>
#include <qfiledlg.h> 
#include <qgrpbox.h> 

// KDE headers
#include <kfm.h>
#include <kapp.h>
#include <kurl.h>
#include <drag.h>
#include <kprogress.h>
#include <kmenubar.h>
#include <kmsgbox.h>
#include <ktopwidget.h> 

// kpackage.headers
#include "managementWidget.h"
#include "installationWidget.h"
#include "pkgInterface.h"

class Search;

class KPACKAGE : public QWidget
{
  Q_OBJECT;

  ///////////// METHODS ------------------------------------------------------
public:
  KPACKAGE(KConfig *_config, QWidget *parent=0, const char *name=0);
  // Constructor

  ~KPACKAGE();
  // Destructor

    enum { OPEN_READWRITE 	= 1, 
	   OPEN_READONLY 	= 2, 
	   OPEN_INSERT 		= 4 };

  void setStatus(const char *s);
  // this sets the status bar's string to s

  void setPercent(int x);
  // this set the status bar's progress to x

  char *getStatus();
  // this gets the current status string on the status bar

  void setMode(int newmode, pkgInterface *type, int refresh);
  // This sets the mode to newmode and updates the display accordingly.

  void installPackage(const char *name, pkgInterface *type);
  // this installs a package

  pkgInterface *pkType(const char *fname);
  // find type of package

  void KPACKAGE::rmtemp();
  // remove temp file

  void openNetFile(const char *);
  // open a file given a URL

protected:
  void resizeEvent(QResizeEvent *re);
  // This is called when the widget is resized

private:

  void setupDropzone();
  // This sets up the DND drop zone

  void setupModeWidgets();
  // This sets up the mode widgets (ie management/installation widgets)

  void setupStatusBar();
  // This sets up the status bar

  void arrangeWidgets();
  // This arranges the widgets in the window (should be called after a
  // resize event)

  QFileDialog* getFileDialog(const char* captiontext);

  Search *srchdialog;

  ///////////// SLOTS --------------------------------------------------------
public slots:
  void dropAction(KDNDDropZone *);
  // This is called when a URL has been dropped in the drop zone

  void modeFinished(int mode, pkgInterface *interface, int refresh);
  // This is called when the mode `mode' has finished.  KPACKAGE should
  // then change modes appropriately

  void slotKFMFinished();
  // fm fetch completed

  void fileOpen();
  // This is called when File->Open is selected from the menu

  void fileOpenUrl();
  // menu item FIle->OpenUrl

  void find();
  // search for package

  void fileQuit();
  // This is called when File->Quit is selected from the menu

  void helpAbout();
  // This is called when Help->About is selected from the menu

  void helpHelp();
  // This is called when Help->Help is selected from the menu

  ///////////// SIGNALS ------------------------------------------------------

  ///////////// DATA ---------------------------------------------------------
public:

  enum { Management, Installation } ; 
  // Widget modes

  KConfig *config;
  // pointer to kconfig object

  managementWidget *management;
  // management widget

  QString save_url;
  // save the URL entered

private:
  int mode;
  // Widget mode

  // Menu item identifiers

  QFrame *statusbar;	    
  // the status bar

  KProgress *processProgress;
  // Progress bar for showing progress

  QLabel *status;
  // The actual status

  KDNDDropZone *dropzone;
  // The dropzone where URLs can be dropped

  QFileDialog *file_dialog;
  /// If we load a file from the net this is the corresponding URL

  QString netFile;
  QString tmpFile;
  // File names for fetching from network

  KFM * kfm;
  // object for net fetchin

};

class KPKG : public KTopLevelWidget
{
  Q_OBJECT;

public:

  KPKG(KConfig *_config,  const char *name=0);
  // Constructor

  ~KPKG();
  // Destructor

  void add_recent_file(const char* newfile);
  // keep list of files accessed

  void readSettings();
  void writeSettings();
  // write and write settings

  void saveProperties(KConfig *config);
  void readProperties(KConfig *config);
  // save and read restart sstate

  void disableMenu();
  void enableMenu();	
  // enable/deisable menu elements

  KMenuBar *menubar;
  QPopupMenu *filemenu, *helpmenu, *recentpopup;
  // The menu bar and its popups

  KPACKAGE *kp;
  // The part under the menu bar

  bool prop_restart;
  // indicates a restart from saved state

  QStrList urlList;
  // For multiple drops save list of URL's for processing

private:
  void setupMenu();
  // This sets up the menubar

  QStrList recent_files;

public slots:

    void openRecent(int);

protected:
    void closeEvent ( QCloseEvent *e);
};

class Search : public QDialog
{
  Q_OBJECT

public:

  Search ( QWidget *parent = 0, const char *name=0);
  ~Search(); 
  void Search::resizeEvent(QResizeEvent *);

private:
    QPushButton *ok, *cancel;
    QCheckBox *substr;
    QCheckBox *wrap;
    QGroupBox *frame1;
    QLineEdit *value;

signals:
    void search_signal();
    void search_done_signal();

public slots:
    void done_slot();
    void ok_slot();
};

extern KPKG *kpkg;

enum {DEBi, RPMi};
extern pkgInterface *kpinterface[];
extern const int kpinterfaceN;

extern void KpMsg(const char *lab, const char *format, const char *str, bool stop);
extern void KpMsgE(const char *format, const char *str, bool stop);
#endif



