////////////////////////////////////////////////////////////////////////
// $Id: managementWidget.h,v 1.4 1997/04/01 10:52:26 root Exp $	
// File  : managementWidget.h
// Author: Damyan Pepper
//
// This widget is used to provide the management mode of ksetup.
// There are two subwidgets; firstly a tree list showing all the
// currently installed packages and secondly a display of the currently
// selected package's properties.
//
// There are also some control buttons which allow the currently
// selected package to be uninstalled or verified.
///////////////////////////////////////////////////////////////////////

#ifndef MANAGEMENTWIDGET_H
#define MANAGEMENTWIDGET_H

#include "../config.h"
// Standard Headers

// Qt Headers
#include <qframe.h>
#include <qpushbt.h>
#include <qlist.h>
#include <qstring.h>
#include <qlayout.h>

// KDE headers

// ksetup headers
#include <ktreeview.h>
#include "packageInfo.h"
#include "packageDisplay.h"

class MyTreeList : public KTreeView
{
public:
    MyTreeList(QWidget *parent = 0,
               const char *name = 0,
               WFlags f = 0) :
    KTreeView(parent , name , f) {};
    void setTopCell(int c) { KTreeView::setTopCell(c); }
    bool rowIsVisible(int c) { return KTreeView::rowIsVisible(c); }
};

class managementWidget : public QFrame
{
  Q_OBJECT;

  ///////////// METHODS ------------------------------------------------------
public:
  managementWidget(QWidget *parent=0, const char *name=0);
  // Constructor

  ~managementWidget();
  // Destructor

  int search(char *str, bool subStr, bool wrap);
  // search for a package in tree

protected:
  void resizeEvent(QResizeEvent *re);
  // This is called when the widget is resized

private:
  void setupWidgets();
  // This sets up the sub-widgets

  void arrangeWidgets();
  // This arranges the widgets in the window (should be called after a
  // resize event)

  ///////////// SLOTS   ------------------------------------------------------
public slots:
  void collectData(bool refresh);
  // This collects data about all the packages installed.
  // The list tree is filled with this data.  Whenever something happens
  // that requires data to be (re)collected a signal connected to this slot
  // should be emitted.  This function can also be called directly.

  void rebuildListTree();
  // This rebuilds the list tree.  This would normally be called if the
  // data contained about the packages has been changed (e.g. a verification
  // failed / succeeded).

private slots:
  void packageHighlighted(int index);
  // This is called when a package has been highlighted in the list tree

  void uninstallClicked();
  // This is called when the uninstall button has been clicked

  ///////////// SIGNALS ------------------------------------------------------

signals:
  void changePackage(packageInfo *p);

  ///////////// DATA ---------------------------------------------------------
private:
  packageInfo *package;

  QPushButton *uninstallButton;
  // This button is used to uninstall the selected package

  MyTreeList *treeList;
  // This is the tree list where all the packages / groups are displayed

  packageDisplayWidget *packageDisplay;
  // This widget displays the package info / file-list

  QBoxLayout *whole, *rightpanel, *buttons;
  // These are the geometry managers

  int searchChild(KTreeViewItem *it, char *str, bool subStr, bool wrap);
  // recurse thru the display tree looking for 'str'

  KTreeViewItem *citem;
  // global used by searchChild to start from current position

public:
  LpackageInfo *installedPackages;
};

#endif



