/////////////////////////////////////////////////////////////////////////
// $Id: packageDisplay.h,v 1.4 1997/04/01 10:52:26 root Exp $	
// File  : packageDisplay.h
// Author: Damyan Pepper
//
// This widget is used to display information and the file list of a
// package.
//
// Package information will be displayed using (another) sub-widget
// that is inherited from a QTableView.
//
// The file list will be displayed using a tree list.
//
// The widget is mainly a QTabDialog with two tabs: Info and FileList.
// The Info tab is the default one.
//
// It may prove that the tab dialog idea doesn't work. Something else
// will have to be done in that case!!!
/////////////////////////////////////////////////////////////////////////

#ifndef PACKAGEDISPLAY_H
#define PACKAGEDISPLAY_H
#include "../config.h"

// Qt Headers
#include <qframe.h>
#include <qtabbar.h>

// ksetup headers
#include "packageInfo.h"
#include <ktreeview.h>
#include "packageProperties.h"

class packagePropertiesWidget;

class packageDisplayWidget : public QFrame
{
  Q_OBJECT;

  ///////////// METHODS ------------------------------------------------------
public:
  packageDisplayWidget(QWidget *parent=0, const char *name=0);
  // Constructor

  ~packageDisplayWidget();
  // Destructor

protected:
  void resizeEvent(QResizeEvent *re);
  // This is called when the widget is resized

private:
  void setupWidgets();
  // This sets up the sub-widgets

  void arrangeWidgets();
  // This arranges the widget in the window (should be called after a
  // resize events

  void updateFileList();
  // This updates the file list to match that found with the currently
  // selected package
  
  ///////////// SLOTS --------------------------------------------------------
public slots:
  void changePackage(packageInfo *p);

  void tabSelected(int tab);

  void openBinding(int);

  ///////////// SIGNALS ------------------------------------------------------

  ///////////// DATA ---------------------------------------------------------
private:
  packageInfo *package;
  // the currently selected package

  QTabBar *tabbar;
  // The tab bar

  KTreeView *fileList;
  // This holds the file list (and is used as a page on the tab dialog)

  QPixmap *tick, *cross, *question, *blank;
  // The pixmaps for the filelist


  packagePropertiesWidget *packageProperties;
  // This displays the package properties (and is used as a page on the
  // tab dialog)

};
#endif
