///////////////////////////////////////////////////////////////////////////////
// $Id: packageInfo.cpp,v 1.5 1997/04/01 10:51:49 root Exp $	
// File  : packageInfo.cpp
// Author: Damyan Pepper
//
// See packageInfo.h for more information.
///////////////////////////////////////////////////////////////////////////////

#include "../config.h"
// Standard headers
#include <stdlib.h>

// KDE headers
#include <kapp.h>

// ksetup headers
#include "packageInfo.h"
#include "pkgInterface.h"
#include "utils.h"

extern KIconLoader  *globalKIL; 

// Global pixmap for 
QPixmap *pict = NULL;

// Constructor -- get the pixmap
packageInfo::packageInfo(QDict<QString> *_info, pkgInterface *type)
{
  interface = type;

  info = _info;
  item = NULL;
  bad_install = FALSE;

  folder = globalKIL->loadIcon("mini/folder.xpm");
}

// Another constructor, for a packge with a filename
packageInfo::packageInfo(QDict<QString> *_info, QString _filename)
{
  info = _info;
  filename = _filename;
  item = NULL;
}

packageInfo::~packageInfo()
{
   delete info;
}

// Return a property
QString *packageInfo::getProperty(char *property)
{
  return info->find(property);
}

// Return the property dictionary
QDict<QString> *packageInfo::getDict()
{
  return info;
}

// Set the file name
void packageInfo::setFilename(const char *f)
{
  filename = f;
}

// Get the filename
QString packageInfo::getFilename()
{
  return filename;
}


// Place the package in a KTreeView

//////////////////////////////////////////////////////////////////////
////// black magic code follows -- need a **sensible** implementation
////// of a tree list and then I'll document it!
//////////////////////////////////////////////////////////////////////

int packageInfo::place(KTreeView *tree)
{
  KTreeViewItem *search = tree->itemAt(0), *parent=NULL;
  QString qtmp, tmp;
  qtmp = interface->head.data();
  qtmp += getProperty("group")->data();
  char *gstructc = strdup(qtmp.data());
  char *gstruct = gstructc;
  int index;
  int cnt = 0;

  KPath thePath;
  thePath.setAutoDelete(true);
  KPath path;
  path.setAutoDelete(true);

  while(*gstruct)			// get path to parent of this item
    {
      char *gtrav = getGroup(&gstruct);
      KTreeViewItem *group;

      if( search && (group=findGroup(gtrav, search)) )
	 {
	   search = group->getChild();
	   parent = group;
	 }
      else
	{
	  group = new KTreeViewItem(gtrav,folder);
	  if (parent)
	    index = tree->itemRow(parent);
	  else
	    index = 0;
	  
	  path.clear();
	  tree->itemPath(index,path);
	  if(path.count() && cnt > 0) {
	    tree->appendChildItem(group, path);
	  } else
	    tree->insertItem(group);
	  search = NULL;
	  parent = group;
	}
      tree->expandItem(tree->itemRow(parent));
      thePath.clear();
      tree->itemPath(tree->itemRow(parent),thePath);   
      cnt++;
    }

  if(item)
    delete item;

  if (info->find("size")) {
    tmp =  *info->find("name") + "\t    " +
      info->find("size")->stripWhiteSpace().data();
    if (interface->sizeTruncate)
      tmp.truncate(tmp.length() - interface->sizeTruncate);
    tmp += "K";
  } else {
    tmp = *info->find("name") + "\t";
  }
  if (!bad_install)
    item = new KTreeViewItem(tmp,*interface->pict);
  else
    item = new KTreeViewItem(tmp,*interface->bad_pict);

  tree->appendChildItem(item, thePath);
  tree->collapseItem(tree->itemRow(parent));
  free(gstructc);
  return 0;//0=success
}

//////////////////////////////////////////////////////////////////////

// Get the KTreeViewItem
KTreeViewItem *packageInfo::getItem()
{
  return item;
}

/////////////////////////////////////////////////////////////////////
LpackageInfo::LpackageInfo()
{
  setAutoDelete(TRUE);
}

LpackageInfo::~LpackageInfo()
{
}

//////////////////////////////////////////////////////////////////////////////
int LpackageInfo::compareItems ( GCI item1, GCI item2)
{
  int n;
  n = strcmp(((packageInfo *)item1)->info->find("group")->data(),
    ((packageInfo *)item2)->info->find("group")->data());
  if (n != 0)
    return n;
  else
    return strcmp(((packageInfo *)item1)->info->find("name")->data(),
		  ((packageInfo *)item2)->info->find("name")->data());
}

