/////////////////////////////////////////////////////////////////////////////////
// File  : pkginstallOptions.cpp
// Author: Damyan Pepper
//

#include "../config.h"
// qt headers
#include <qlabel.h>

// ksetup headers
#include "pkginstallOptions.h"
#include "pkginstallOptions.moc"
#include "debInterface.h"
#include "kpackage.h"


// Constructor -- set the frame style, and setup the sub widgets
pkginstallOptionsWidget::pkginstallOptionsWidget(QWidget *parent,
				    param *pars, const char *name)
  : QFrame(parent,name)
{
  setFrameStyle(QFrame::Raised | QFrame::Panel);

  setupWidgets(pars);
  package = 0;
}

// Destructor
pkginstallOptionsWidget::~pkginstallOptionsWidget()
{
  int i;
  for (i = 0; i < bnumber; i++) {
    delete(Boxs[i]);
  }
}

// Set up the sub-widgets
void pkginstallOptionsWidget::setupWidgets(param *pars)
{
  int i;

  // Create widgets
  QLabel *title = new QLabel(klocale->translate("Install Package"), this);
  title->setFont(QFont("Helvetica",18, QFont::Bold));
  title->setAutoResize(TRUE);
  title->update();

  installButton = new QPushButton(klocale->translate("Install"),this);
  cancelButton = new QPushButton(klocale->translate("Cancel"),this);
  // count number of buttons
  for (i = 0; pars[i].name != 0; i++) {
    bnumber = i;
  }
  bnumber++;

  Boxs = new QCheckBox *[bnumber];
  for (i = 0; i < bnumber ; i++) {
    Boxs[i] = new QCheckBox(pars[i].name, this);
    Boxs[i]->setChecked(pars[i].init);
  }
  
  // Connections
  connect(installButton,SIGNAL(clicked()),SLOT(pkginstallButtonClicked()));
  connect(cancelButton,SIGNAL(clicked()),SLOT(cancelButtonClicked()));

  // Do the layout
  layout = new QBoxLayout(this,QBoxLayout::TopToBottom,bnumber);
  QBoxLayout *buttons = new QBoxLayout(QBoxLayout::LeftToRight);

  layout->addWidget(title,1);
  layout->addStretch(1);
  for (i = 0; i < bnumber; i++) {
    layout->addWidget(Boxs[i],1);
  }
  layout->addStretch(2);
  layout->addLayout(buttons);

  buttons->addStrut(20);
  buttons->addWidget(installButton,2);
  buttons->addStretch(1);
  buttons->addWidget(cancelButton,2);

  layout->activate();
}

// Set the current package that is being worked with
void pkginstallOptionsWidget::setPackage(packageInfo *p)
{
  package = p;
}

// install button has been clicked....so install the package
void pkginstallOptionsWidget::pkginstallButtonClicked()
{
  // Disable the buttons
  int i, r;

  installButton->setEnabled(FALSE);
  cancelButton->setEnabled(FALSE);
  for (i = 0; i < bnumber; i++) {
    Boxs[i]->setEnabled(FALSE);
  }

  // Collect data from check boxes
  int installFlags = 0;
  
  for (i = 0; i < bnumber; i++) {
    installFlags |= (Boxs[i]->isChecked()) << i;
  }	


  //  interfaceFlags|=INSTALL_PERCENT;

  r = package->interface->install(installFlags, package);

  // enable the buttons
  installButton->setEnabled(TRUE);
  cancelButton->setEnabled(TRUE);
  for (i = 0; i < bnumber; i++) {
    Boxs[i]->setEnabled(TRUE);
  }

  // Emit the finished signal to indicate that installation is complete
  if (!r)
    emit finished(1);
}

// Cancel button has been clicked -- finish without doing anything
void pkginstallOptionsWidget::cancelButtonClicked()
{
  emit finished(0);
}
