// 	$Id: rpmInterface.cpp,v 1.3 1997/04/02 09:38:06 root Exp $	
#include "../config.h"
#ifdef HAVE_RPM
extern "C"
{
#include "rpmlib_new.h"
#include "rpmVerify.h"
#include "rpmInstall.h"

#include <rpm/dbindex.h>
}

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>		// for O_RDONLY

#include <time.h>		// for localtime

#include "rpmInterface.h"
#include "packageInfo.h"
#include "kpackage.h"
#include "rpmutils.h"

static param pinstall[] =  {
  {"Upgrade",TRUE,FALSE},
  {"Replace Files",FALSE,FALSE},
  {"Replace Packages",TRUE,FALSE},
  {"Check Dependencies",TRUE,TRUE},
  {"Test (do not install)",FALSE,FALSE},
  {0,FALSE,FALSE}
};

static param puninstall[] =  {
  {"Use Scripts",TRUE,FALSE},
  {"Check Dependencies",TRUE,TRUE},
  {"Test (do not uninstall)",FALSE,FALSE},
  {0,FALSE,FALSE}
};

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
RPM::RPM()
{
  char * rcfile = NULL;
  char *arch = NULL;
  char *os = NULL;
  int building = 0;
  int showrc = 0;  
  sizeTruncate = 3;
  head = "RPM/";

  pinstall[0].name = klocale->translate("Upgrade");
  pinstall[1].name = klocale->translate("Replace Files");
  pinstall[2].name = klocale->translate("Replace Packages");
  pinstall[3].name = klocale->translate("Check Dependencies");
  pinstall[4].name = klocale->translate("Test (do not install)");

  puninstall[0].name = klocale->translate("Use Scripts");
  puninstall[1].name = klocale->translate("Check Dependencies");
  puninstall[2].name = klocale->translate("Test (do not uninstall)");

  icon = "rpm.xpm";
  pict = new QPixmap();
  *pict = globalKIL->loadIcon(icon); 

  /* reading this early makes it easy to override */
  if(rpmReadConfigFiles(rcfile, arch, os, building))  
  {
   KpMsgE( klocale->translate("Cann't read RPM config files\n"),"",TRUE);
    return;
  }
  if(showrc) 
    {
      rpmShowRC(stdout);
      exit(0);
    }
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
RPM::~RPM()
{
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
param *RPM::initinstallOptions()
{
  return &(pinstall[0]);
}

param *RPM::inituninstallOptions()
{
  return &(puninstall[0]);
}

bool RPM::isType(char *buf)
{
  if ((unsigned char)buf[0] == 0355 && (unsigned char)buf[1] == 0253 &&
      (unsigned char)buf[2] == 0356 && (unsigned char)buf[3] == 0333 ) {
    return true;
  } else
    return false;
}


void RPM::listInstalled(LpackageInfo *pki)
{
  int percentage;
  int offset;
  Prpmdb db;
  Header h;
  char *prefix = "";

  kpkg->kp->setStatus(klocale->translate("Querying RPM database for installed packages"));
  percentage=0;
  kpkg->kp->setPercent(percentage);
  
  if ( rpmdbOpen(prefix,&db, O_RDONLY, 0644) )
    return;
  
  offset = rpmdbFirstRecNum(db);
  while (offset)
    {
      percentage+=5;
      if(percentage > 100) percentage=0;
      kpkg->kp->setPercent(percentage);
      h = rpmdbGetRecord(db, offset);
      if (!h) 
	{
	  KpMsgE(klocale->translate("could not read database record\n"),"",TRUE);
	  return;
	}
      
      pki->inSort(RPM::collectInfo(h));
      
      headerFree(h);
      offset = rpmdbNextRecNum(db, offset);
    }
  
  kpkg->kp->setPercent(100);
  rpmdbClose(db);
  kpkg->kp->setStatus("");
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// mode: i = query installed    u = query uninstalled
packageInfo *RPM::getPackageInfo(char mode, const char *name)
{
  Prpmdb db;
  int fd, rc=0, isSource;
  Header h;
  char *prefix = ""; // this is the root directory -- should be configured!
  packageInfo *pki = 0;;
  
  switch(mode)
    {
      ////////////////////////////////////////////////////////////////////////
      // query an installed package!
    case 'i':
      if (rpmdbOpen(prefix,&db, O_RDONLY, 0644) )
	exit(3);

      dbiIndexSet matches;

      rc = findPackageByLabel(db, name, &matches);
      if(rc==1)
	KpMsgE(klocale->translate("Package %s is not installed\n"),name,TRUE);
      else if(rc==2)
	KpMsgE(klocale->translate("Error looking for package %s\n"),name,TRUE);
      else
	{
	  int i;
	  for(i=0; i<matches.count; i++)
	    {
	      if(matches.recs[i].recOffset)
		{
		  h = rpmdbGetRecord(db, matches.recs[i].recOffset);
		  if(!h)
		    KpMsgE(  klocale->translate("Could not read database record\n"),"",TRUE);
		  else
		    {
		      pki = RPM::collectInfo(h);
		      headerFree(h);
		    }
		}
	    }

	  collectDepends(pki, pki->getProperty("name")->data(),VERIFY_PACKAGE);

	  dbiFreeIndexRecord(matches);
	  rpmdbClose(db);
	}
      break;

      ////////////////////////////////////////////////////////////////////
      // query an uninstalled package      
    case 'u':
      if((fd = open(name, O_RDONLY)) < 0)
	{
	  KpMsgE(klocale->translate("opening %s\n"),name,TRUE);
	  return pki;
	}
      if(fd>=0)
	{
	  rc = rpmReadPackageHeader(fd, &h, &isSource, NULL, NULL);
	  close(fd);
	}
      switch(rc)
	{
	case 0:
	  if(!h)
	    KpMsgE(klocale->translate("Old format source packages cannot be queried!\n"),"",TRUE);
	  else
	    {
	      pki = RPM::collectInfo(h);
	      headerFree(h);
	      collectDepends(pki, name, VERIFY_RPM);
	    }
	  break;
	case 1:
	  KpMsgE(klocale->translate("%s does not appear to be a RPM package\n"),name,TRUE);
	  // fallthrough
	case 2:
	  KpMsgE(klocale->translate("Query of %s failed!\n"),name,TRUE);
	  return 0;
	}
      break;
    }
  return pki;
}


//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
#define DO(x,z)                                                  \
{                                                                \
  char *k;                                                       \
  headerGetEntry(h, z, &type, (void **) &k, &count);             \
  if(k)                                                          \
    a->insert(x, new QString(k));                                \
}

#define DO2(x,z)                                                 \
{                                                                \
  int_32 k;                                                      \
  char *k2;                                                      \
  headerGetEntry(h, z, RPM_INT32_TYPE, (void **) &k, &count);    \
  if(k)                                                          \
    {                                                            \
      sprintf(k2,"%d",k);                                        \
      a->insert(x, new QString(k2));                             \
      printf(" %s -- %s\n",x,k);                                 \
    }                                                            \
}                                

packageInfo *RPM::collectInfo(Header h)
{
  int_32 type,count;
  time_t dateint;   struct tm * tstruct;   char buf[100];
  
  QDict<QString> *a = new QDict<QString>;
  a->setAutoDelete(TRUE);      

  DO("name",RPMTAG_NAME);
  DO("version",RPMTAG_VERSION);
  DO("release",RPMTAG_RELEASE);
  DO("summary",RPMTAG_SUMMARY);
  DO("group",RPMTAG_GROUP);
  char *k;


  // Get nice description
  headerGetEntry(h, RPMTAG_DESCRIPTION, &type, (void **) &k, &count);
  if(k)
    {
      char *p;
      for(p = k; *p!=0; p++)	// remove newlines
	{
	  if(*p == '\n')
	    {
	      if(*(p+1) == '\n')
		{
		  p++;
		}
	      else
		*p = ' ';
	    }
	}
      a->insert("description",new QString(k));
    }

  // Get nice install time
  headerGetEntry(h, RPMTAG_INSTALLTIME, &type, (void **) &k, &count);
  if(k)
    {
      /* this is important if sizeof(int_32) ! sizeof(time_t) */
      dateint = *(((int_32 *) k) + 0);
      tstruct = localtime(&dateint);      
      strftime(buf, sizeof(buf) - 1, "%c", tstruct);      
      
      a->insert("installtime", new QString(buf));
    }
  
  // Get nice size
  headerGetEntry(h, RPMTAG_SIZE, &type, (void **) &k, &count);
  if(k)
    {
      sprintf(buf,"%d",*((int_32 *) k) );
      
      a->insert("size", new QString(buf));
    }
  
  packageInfo *i = new packageInfo(a,this);
  return i;
}
#undef DO
#undef DO2

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
QList<char> *RPM::getFileList(packageInfo *p)
{

  Prpmdb db;
  int fd, rc=0, isSource;
  Header h;
  char *prefix = ""; // this is the root directory -- should be configured!
  char *name;
  char mode;
  if(p->getFilename().data())
    mode = 'u';
  else
    mode = 'i';

  QList<char> *filelist=NULL;

  switch(mode)
    {
      ////////////////////////////////////////////////////////////////////////
      // query an installed package!
    case 'i':
      name = p->getProperty("name")->data();
      if (rpmdbOpen(prefix,&db, O_RDONLY, 0644) )
	exit(3);

      dbiIndexSet matches;

      rc = findPackageByLabel(db, name, &matches);
      if(rc==1)
	KpMsgE(klocale->translate("Package %s is not installed\n"),name,TRUE);
      else if(rc==2)
	KpMsgE(klocale->translate("Error looking for package %s\n"),name,TRUE);
      else
	{
	  int i;
	  for(i=0; i<matches.count; i++)
	    {
	      if(matches.recs[i].recOffset)
		{
		  h = rpmdbGetRecord(db, matches.recs[i].recOffset);
		  if(!h)
		    KpMsgE(klocale->translate("Could not read database record\n"),"",TRUE);
		  else
		    {
		      filelist = RPM::collectFileList(h);
		      headerFree(h);
		    }
		}
	    }
	  dbiFreeIndexRecord(matches);
	  rpmdbClose(db);
	}
      break;

      ////////////////////////////////////////////////////////////////////
      // query an uninstalled package      
    case 'u':
      name = p->getFilename().data();
      if((fd = open(name, O_RDONLY)) < 0)
	{
	  KpMsgE(klocale->translate("ERROR opening %s\n"),name,TRUE);
	  return filelist;
	}
      if(fd>=0)
	{
	  rc = rpmReadPackageHeader(fd, &h, &isSource, NULL, NULL);
	  close(fd);
	}
      switch(rc)
	{
	case 0:
	  if(!h)
	    KpMsgE(klocale->translate("Old format source packages cannot be queried\n"),"",TRUE);
	  else
	    {
	      filelist = RPM::collectFileList(h);
	      headerFree(h);
	    }
	  break;
	case 1:
	  KpMsgE(klocale->translate("%s does not appear to be a RPM package\n"),name,TRUE);
	  // fallthrough
	case 2:
	  KpMsgE(klocale->translate("Query of %s failed\n"),name,TRUE);
	  return filelist;
	}
      break;
    }
   
  return filelist;
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
QList<char>* RPM::collectFileList(Header h)
{
  int_32 type, count;
  char **fileList;
  QList<char>* files = new QList<char>;

  kpkg->kp->setStatus(klocale->translate("Getting file list"));
  kpkg->kp->setPercent(0);

  if(!headerGetEntry(h, RPMTAG_FILENAMES, &type, (void **) &fileList, &count))
    {
      kpkg->kp->setPercent(100);
      return files;
    }
  else
    {
      int i;
      for(i=0; i<count; i++)
	{
	  kpkg->kp->setPercent( (i/count) * 100);
	  files->append(fileList[i]);
	}
      kpkg->kp->setPercent(100);
      return files;
    }
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
QList<char> *RPM::depends(const char *name, int src)
{
  kpkg->kp->setStatus(klocale->translate("Depends"));
  kpkg->kp->setPercent(0);

  int verifyFlags = 0;
  verifyFlags |= VERIFY_DEPS;

  const char *fls[2];
  fls[0] = name;
  fls[1] = NULL;

  return doVerify("/", (enum verifysources) src, fls, verifyFlags);
  
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
QList<char> *RPM::verify(packageInfo *p, QList<char> *files)
{
  files = files; //silence compiler

  kpkg->kp->setStatus(klocale->translate("Verifying"));
  kpkg->kp->setPercent(0);

  int verifyFlags = 0;
  verifyFlags |= VERIFY_FILES |  VERIFY_SCRIPT |  VERIFY_MD5;

  char *fls[2];
  fls[0] = p->getProperty("name")->data();
  fls[1] = NULL;

  return doVerify("/", VERIFY_PACKAGE, fls, verifyFlags);
  
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
int RPM::uninstall(int uninstallFlags, packageInfo *p)
{
  int interfaceFlags = 0 , uninstFlags = 0, test = 0;;
  char *files[2];
  files[0]=p->getProperty("name")->data();
  files[1]=NULL;

  if ((uninstallFlags>>0 & 1) ^ pinstall[0].invert)
    uninstFlags|=RPMUNINSTALL_NOSCRIPTS;
  if ((uninstallFlags>>1 & 1) ^ pinstall[1].invert)
    interfaceFlags|=UNINSTALL_NODEPS;
  if ((uninstallFlags>>2 & 1) ^ pinstall[2].invert) {
    uninstFlags|=RPMUNINSTALL_TEST;
    test = 1;
  }

  return (doUninstall("/", files, uninstFlags, interfaceFlags) || test);
  
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
int RPM::install(int installFlags, packageInfo *p)
{
  int interfaceFlags = 0 , instFlags = 0, test=0;
  char *files[2];
  files[0]=p->getFilename().data();
  files[1]=NULL;

  if ((installFlags>>0 & 1) ^ pinstall[0].invert)
    instFlags |= RPMINSTALL_UPGRADE;
  if ((installFlags>>1 & 1) ^ pinstall[1].invert)
    instFlags|=RPMINSTALL_REPLACEFILES;
  if ((installFlags>>2 & 1) ^ pinstall[2].invert)
    instFlags|=RPMINSTALL_REPLACEPKG;
  if ((installFlags>>3 & 1) ^ pinstall[3].invert)
    interfaceFlags|=INSTALL_NODEPS;
  if ((installFlags>>4 & 1) ^ pinstall[4].invert) {
    instFlags|=RPMINSTALL_TEST;
    test = 1;
  }
  interfaceFlags|=INSTALL_PERCENT;

  return (doInstall("/", files, 0, instFlags,
		interfaceFlags) || test);
}
#endif
